package com.lilosoft.api.service.complexWindow;

import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.lilosoft.api.bean.receive.FileDataItem;
import com.lilosoft.complex.Rest;
import com.lilosoft.complex.matter.model.req.*;
import com.lilosoft.complex.matter.model.rsp.AcceptRspInfo;
import com.lilosoft.complex.matter.model.rsp.LoginRspInfo;
import com.lilosoft.complex.matter.model.rsp.MatterListInfo;
import com.lilosoft.complex.matter.model.rsp.MatterRespInfo;
import com.lilosoft.complex.matter.model.rsp.matter.ChildrenItem;
import com.lilosoft.complex.matter.model.rsp.matter.MatterWorkInfo;
import com.lilosoft.complex.matter.service.IComplexMatterService;
import com.lilosoft.core.enums.TypeEnum;
import com.lilosoft.core.enums.YesNoEnum;
import lombok.extern.slf4j.Slf4j;
import okhttp3.ResponseBody;
import org.springframework.beans.BeanUtils;
import org.springframework.util.ObjectUtils;
import retrofit2.Response;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 危险化学品经营许可法人变更
 *
 * @author:
 * @date: 2023/7/31 15:15
 */
@Slf4j
public class WXHXPJYXKFRBG extends ComplexWindowAbstract {


    private IComplexMatterService complexMatterService;

    public static String matterName = "从事剧毒、易制爆、汽油加油站、专用危化品仓储的企业，中央企业所属省级";


    public WXHXPJYXKFRBG(String type) {
        super(type);
        this.complexMatterService = SpringUtil.getBean(IComplexMatterService.class);

    }

    @Override
    String getType() {
        return TypeEnum.危险化学品经营许可法人变更.getValue();
    }

    @Override
    public void doHandleWork(CommonData commonData) {
        //获取数据资源 todo

        //下载附件
        List<FileDataItem> fileCollect = commonData.getFileData().stream().map(item -> {
            FileDataItem fileDataItem = new FileDataItem();
            BeanUtils.copyProperties(item, fileDataItem);
            String downloadPath = commonData.getRobertServerUrl() + item.getFileUrl();

            byte[] bytes = HttpUtil.downloadBytes(downloadPath);
            fileDataItem.setBytes(bytes);
            return fileDataItem;
        }).collect(Collectors.toList());

        //通过名称查询
        DeviceLoginReq deviceLoginReq = new DeviceLoginReq();
        deviceLoginReq.setDeviceLogo(commonData.getDeviceLogo());
        Rest<LoginRspInfo> loginRspInfoRest = complexMatterService.deviceLogin(deviceLoginReq);
        if (loginRspInfoRest.getCode() == YesNoEnum.YES.getValue()) {
            //设备登录成功 上传附件
            fileCollect.stream().forEach(item -> {
                Rest<String> uploadFileRest = complexMatterService.uploadFile(commonData.getDeviceLogo(), item.getBytes(), item.getFileName());
                if (uploadFileRest.getCode() == YesNoEnum.YES.getValue()) {
                    item.setDownloadUrl(uploadFileRest.getData());
                }
            });

            //通过事项查询eventId
            ImplementationReq implementationReq = new ImplementationReq();
            implementationReq.setImplementName(matterName);
            Rest<MatterListInfo> matterRest = complexMatterService.findMatters(commonData.getDeviceLogo(), implementationReq);
            if (matterRest.getCode() == YesNoEnum.YES.getValue()) {
                List<MatterRespInfo> matterListInfos = matterRest.getData().getRecords();
                if (!ObjectUtils.isEmpty(matterListInfos)) {
                    MatterRespInfo matterRespInfo = matterListInfos.get(0);
                    String eventId = matterRespInfo.getEventId();
                    String eventObjectType = matterRespInfo.getEventObjectType();
                    AcceptReq acceptReq = new AcceptReq();

                    //查询办理事项情形
                    Rest<MatterWorkInfo> matterWorkInfoRest = complexMatterService.doWorkMatterSearch(commonData.getDeviceLogo(), eventId);
                    if (matterWorkInfoRest.getCode() == YesNoEnum.YES.getValue()) {
                        MatterWorkInfo matterWorkInfo = matterWorkInfoRest.getData();
                        ChildrenItem childrenItem = matterWorkInfo.getChildren().stream().filter(f -> "变更法人".equals(f.getLabel())).findFirst().orElseGet(() -> null);

                        if (!ObjectUtils.isEmpty(childrenItem)) {
                            //构建受理材料附件
                            List<MaterialsItem> materialsItemList = childrenItem.getMaterials().stream().map(item -> {
                                MaterialsItem materialsItem = new MaterialsItem();
                                String name = item.getName();
                                if (StrUtil.contains(name, "变更申请表")) {
                                    matchUploadFile(fileCollect, materialsItem, "申请书");
                                } else if (StrUtil.contains(name, "安全资格证书")) {
                                    matchUploadFile(fileCollect, materialsItem, "合格证");
                                } else if (StrUtil.contains(name, "化学品经营许可证")) {
                                    matchUploadFile(fileCollect, materialsItem, "许可证");
                                }
                                materialsItem.setMaterialForm(item.getFromId());
                                materialsItem.setMaterialId(item.getId());

                                return materialsItem;

                            }).collect(Collectors.toList());
                            acceptReq.setMaterials(materialsItemList);
                        }
                    }
                    //提交响应的受理


                    List<HandlingsItem> handlingsItemList = new ArrayList<>();
                    HandlingsItem handlingsItem = new HandlingsItem();
                    handlingsItem.setEventId(eventId);
                    handlingsItem.setObjectType(eventObjectType);
                    handlingsItemList.add(handlingsItem);
                    acceptReq.setHandlings(handlingsItemList);


                    //申请人信息
                    ApplicantLeagal applicantLeagal = new ApplicantLeagal();
                    applicantLeagal.setPhone(commonData.getApplicantPhone());
                    applicantLeagal.setApplicantSex(commonData.getApplicantSex());
                    applicantLeagal.setDocumentId(commonData.getApplicantDocumentId());
                    applicantLeagal.setApplicantName(commonData.getApplicantName());
                    applicantLeagal.setApplicantIdNumber(commonData.getApplicantIdNumber());

                    acceptReq.setApplicantLegal(applicantLeagal);

                    //法人信息
                    LegalProxy legalProxy = new LegalProxy();
                    legalProxy.setLegalName(commonData.getLegalName());
                    legalProxy.setLegalIdNumber(commonData.getLegalIdNumber());
                    legalProxy.setDocumentId(commonData.getLegalDocumentId());
                    acceptReq.setLegalProxy(legalProxy);


                    Rest<AcceptRspInfo> acceptRest = complexMatterService.accept(commonData.getDeviceLogo(), acceptReq);

                    log.info("受理返回==>{}", JSON.toJSONString(acceptRest));
                    if (acceptRest.getCode() == YesNoEnum.YES.getValue()) {

                    }
                }


            }

        }

    }

    private static void matchUploadFile(List<FileDataItem> fileCollect, MaterialsItem materialsItem, String fileName) {
        FileDataItem fileDataItem = fileCollect.stream().filter(f -> StrUtil.contains(f.getName(), fileName)).findFirst().orElseGet(() -> null);
        if (!ObjectUtils.isEmpty(fileDataItem)) {
            materialsItem.setUploadFileName(fileDataItem.getFileName());
            materialsItem.setMaterialPath(fileDataItem.getDownloadUrl());
        }
    }
}
