package com.lilosoft.config.interceptor;

import com.google.common.util.concurrent.RateLimiter;
import com.lilosoft.core.annotation.RateLimit;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.concurrent.ConcurrentHashMap;

@Component
public class RateLimitInterceptor extends HandlerInterceptorAdapter {

    //用来存放不同接口的RateLimiter(key为接口名称，value为RateLimiter)
    private ConcurrentHashMap<String, RateLimiter> map = new ConcurrentHashMap<>();

    private RateLimiter rateLimiter;

    @Value("${api.requestLimit}")
    private int requestLimit;

    /**
     *
     * @param request
     * @param response
     * @param handler 访问的目标方法
     * @return
     * @throws Exception
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (handler instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            Method method = handlerMethod.getMethod();
            RateLimit annotation = method.getAnnotation(RateLimit.class);
            String servletPath = request.getServletPath();//注解调用接口方法名，区分不同的限流策略
            int limitNum = annotation == null ? requestLimit : annotation.limitNum();//获取注解每秒加入桶中的token
            //获取rateLimiter
            if(map.containsKey(servletPath)){
                rateLimiter = map.get(servletPath);
            } else {
                map.put(servletPath, RateLimiter.create(limitNum));
                rateLimiter = map.get(servletPath);
            }
            Assert.isTrue(rateLimiter.tryAcquire(), "请求繁忙，请稍后再试");
        }

        return super.preHandle(request, response, handler);
    }
}
